==> trend_etl-1.4.4.tar.gz: trend_etl-1.4.4/trend_etl/transform/comments.py <==
from pyspark.sql import DataFrame
from typing import List, Callable, Any

# from pyspark.sql.column import Column
from pyspark.sql.functions import (
    col,
    lit,
    to_json,
    named_struct,
)
from pyspark.sql.types import StringType, BinaryType

import trend_etl.schema.fields as fields
from trend_etl.functions.index import (
    jobid_2_timestamp,
    reactions_struct,
    parse_facebook_comments,
    conv_timestamp,
    get_created_day,
)


# arguments:
#   archive_df: the dataframe from the archives
#   group_ids: the source ids used in filtering
#   include_flag: whether to use the filtering logic to include/exclude
#   ner_model_fn: apply ner model to anonymize statments
def from_archive_get_bronze_comments_df(
    archive_df: DataFrame,
    group_ids: List[str],
    include_flag: bool,
    ner_model_fn: Callable[[Any], Any],
) -> DataFrame:
    parsed = archive_df.withColumn(
        fields.archive_value_field,
        parse_facebook_comments(col(fields.archive_value_field)),
    )
    message_field = col(f"{fields.archive_value_field}.message")
    created_time_field = col(f"{fields.archive_value_field}.created_time")

    filtered = parsed.where(
        col(f"{fields.archive_value_field}.group_id").isin(group_ids) == include_flag
    )
    anonymized = filtered.withColumn(
        fields.archive_value_field,
        col(fields.archive_value_field).withField(
            "anonymized_message",
            ner_model_fn(message_field),
        ),
    )

    return anonymized.select(
        to_json(col(fields.archive_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit("archive/facebook/comments").alias(fields.bronze_provenance_field),
        jobid_2_timestamp(fields.archive_job_id_field).alias(
            fields.bronze_timestamp_field
        ),
        get_created_day(conv_timestamp(created_time_field)).alias(
            fields.created_day_field
        ),
    )


# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
#   ner_model_fn: apply ner model to anonymize statments
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_comments_df(
    input_df: DataFrame,
    target_topic: str,
    ner_model_fn: Callable[[Any], Any],
) -> DataFrame:
    parsed = input_df.withColumn(
        fields.kafka_value_field,
        parse_facebook_comments(col(fields.kafka_value_field).cast(StringType())),
    )
    message_field = col(f"{fields.kafka_value_field}.message")
    created_time_field = col(f"{fields.kafka_value_field}.created_time")

    anonymized = parsed.withColumn(
        fields.kafka_value_field,
        col(fields.kafka_value_field).withField(
            "anonymized_message",
            ner_model_fn(message_field),
        ),
    )

    return anonymized.select(
        to_json(col(fields.kafka_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_created_day(conv_timestamp(created_time_field)).alias(
            fields.created_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   group_idents: groups uuid idents table
#   post_idents: posts uuid idents table
#   comment_idents: comments uuid idents table
# return: dataframe in the schema of facebook_comments
def from_bronze_get_comments_df(
    streaming_bronze: DataFrame,
    group_idents: DataFrame,
    post_idents: DataFrame,
    comment_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_facebook_comments(
            col(fields.bronze_value_field).cast(StringType())
        ).alias("value"),
        col(fields.bronze_timestamp_field),
    )
    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
    )
    reactions_struct_df = reactions_struct(flattened_df)

    # NOTE: left outer joins (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with group_idents
    group_idents_df = (
        reactions_struct_df.join(
            group_idents.withColumnRenamed(
                fields.idents_group_id_field, fields.idents_doc_id_field
            ).withColumnRenamed(
                fields.idents_source_id_field, fields.idents_group_id_field
            ),
            fields.idents_group_id_field,
            "left_outer",
        )
        .drop(fields.idents_group_id_field)
        .withColumnRenamed(fields.idents_doc_id_field, fields.idents_group_id_field)
    )

    # join with comments_idents to get doc_uuid
    comment_idents_df = group_idents_df.join(
        comment_idents,
        fields.idents_source_id_field,
        "left_outer",
    ).withColumnRenamed(fields.idents_source_id_field, "_source_key_id")

    # join with post_idents to get corresponding posts uuid fk
    assoc_post_df = comment_idents_df.join(
        post_idents.withColumnsRenamed(
            {
                fields.idents_doc_id_field: fields.facebook_comments_post_uuid,
                fields.idents_source_id_field: "post_id",
            }
        ),
        "post_id",
        "left_outer",
    ).withColumnRenamed("post_id", "_source_key_post_id")

    # join with comment_idents to get parent uuid fk
    assoc_parent_df = assoc_post_df.withColumn(
        "_source_key_parent_id", col("parent.id")
    ).join(
        comment_idents.withColumnsRenamed(
            {
                fields.idents_doc_id_field: fields.facebook_comments_parent_uuid,
                fields.idents_source_id_field: "_source_key_parent_id",
            }
        ),
        "_source_key_parent_id",
        "left_outer",
    )

    comments_df = assoc_parent_df.select(
        col(fields.idents_doc_id_field),
        col(fields.idents_group_id_field),
        col(fields.facebook_comments_post_uuid),
        col(fields.facebook_comments_parent_uuid),
        conv_timestamp("created_time").alias(fields.facebook_comments_created_at),
        col("anonymized_message").alias(fields.facebook_comments_text),
        col("permalink_url").alias(fields.facebook_comments_permalink_url),
        col(fields.reactions_struct_field),
        named_struct(
            lit("id"),
            col("_source_key_id"),
            lit("post_id"),
            col("_source_key_post_id"),
            lit("parent_id"),
            col("_source_key_parent_id"),
        ).alias(fields.facebook_comments_source_keys),
        col(fields.bronze_timestamp_field),
    )

    return comments_df

==> trend_etl-1.4.7.tar.gz: trend_etl-1.4.7/trend_etl/transform/comments.py <==
from pyspark.sql import DataFrame
from typing import List, Callable, Any

# from pyspark.sql.column import Column
from pyspark.sql.functions import (
    col,
    lit,
    to_json,
    struct,
)
from pyspark.sql.types import StringType, BinaryType

import trend_etl.schema.fields as fields
from trend_etl.functions.index import (
    jobid_2_timestamp,
    reactions_struct,
    parse_facebook_comments,
    conv_timestamp,
    get_created_day,
)


# arguments:
#   archive_df: the dataframe from the archives
#   group_ids: the source ids used in filtering
#   include_flag: whether to use the filtering logic to include/exclude
#   ner_model_fn: apply ner model to anonymize statments
def from_archive_get_bronze_comments_df(
    archive_df: DataFrame,
    group_ids: List[str],
    include_flag: bool,
    ner_model_fn: Callable[[Any], Any],
) -> DataFrame:
    parsed = archive_df.withColumn(
        fields.archive_value_field,
        parse_facebook_comments(col(fields.archive_value_field)),
    )
    message_field = col(f"{fields.archive_value_field}.message")
    created_time_field = col(f"{fields.archive_value_field}.created_time")

    filtered = parsed.where(
        col(f"{fields.archive_value_field}.group_id").isin(group_ids) == include_flag
    )
    anonymized = filtered.withColumn(
        fields.archive_value_field,
        col(fields.archive_value_field).withField(
            "anonymized_message",
            ner_model_fn(message_field),
        ),
    )

    return anonymized.select(
        to_json(col(fields.archive_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit("archive/facebook/comments").alias(fields.bronze_provenance_field),
        jobid_2_timestamp(fields.archive_job_id_field).alias(
            fields.bronze_timestamp_field
        ),
        get_created_day(conv_timestamp(created_time_field)).alias(
            fields.created_day_field
        ),
    )


# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
#   ner_model_fn: apply ner model to anonymize statments
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_comments_df(
    input_df: DataFrame,
    target_topic: str,
    ner_model_fn: Callable[[Any], Any],
) -> DataFrame:
    parsed = input_df.withColumn(
        fields.kafka_value_field,
        parse_facebook_comments(col(fields.kafka_value_field).cast(StringType())),
    )
    message_field = col(f"{fields.kafka_value_field}.message")
    created_time_field = col(f"{fields.kafka_value_field}.created_time")

    anonymized = parsed.withColumn(
        fields.kafka_value_field,
        col(fields.kafka_value_field).withField(
            "anonymized_message",
            ner_model_fn(message_field),
        ),
    )

    return anonymized.select(
        to_json(col(fields.kafka_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_created_day(conv_timestamp(created_time_field)).alias(
            fields.created_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   group_idents: groups uuid idents table
#   post_idents: posts uuid idents table
#   comment_idents: comments uuid idents table
# return: dataframe in the schema of facebook_comments
def from_bronze_get_comments_df(
    streaming_bronze: DataFrame,
    group_idents: DataFrame,
    post_idents: DataFrame,
    comment_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_facebook_comments(
            col(fields.bronze_value_field).cast(StringType())
        ).alias("value"),
        col(fields.bronze_timestamp_field),
    )
    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
    )
    reactions_struct_df = reactions_struct(flattened_df)

    # NOTE: left outer joins (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with group_idents
    group_idents_df = (
        reactions_struct_df.join(
            group_idents.withColumnRenamed(
                fields.idents_group_id_field, fields.idents_doc_id_field
            ).withColumnRenamed(
                fields.idents_source_id_field, fields.idents_group_id_field
            ),
            fields.idents_group_id_field,
            "left_outer",
        )
        .drop(fields.idents_group_id_field)
        .withColumnRenamed(fields.idents_doc_id_field, fields.idents_group_id_field)
    )

    # join with comments_idents to get doc_uuid
    comment_idents_df = group_idents_df.join(
        comment_idents,
        fields.idents_source_id_field,
        "left_outer",
    ).withColumnRenamed(fields.idents_source_id_field, "_source_key_id")

    # join with post_idents to get corresponding posts uuid fk
    assoc_post_df = comment_idents_df.join(
        post_idents.withColumnsRenamed(
            {
                fields.idents_doc_id_field: fields.facebook_comments_post_uuid,
                fields.idents_source_id_field: "post_id",
            }
        ),
        "post_id",
        "left_outer",
    ).withColumnRenamed("post_id", "_source_key_post_id")

    # join with comment_idents to get parent uuid fk
    assoc_parent_df = assoc_post_df.withColumn(
        "_source_key_parent_id", col("parent.id")
    ).join(
        comment_idents.withColumnsRenamed(
            {
                fields.idents_doc_id_field: fields.facebook_comments_parent_uuid,
                fields.idents_source_id_field: "_source_key_parent_id",
            }
        ),
        "_source_key_parent_id",
        "left_outer",
    )

    comments_df = assoc_parent_df.select(
        col(fields.idents_doc_id_field),
        col(fields.idents_group_id_field),
        col(fields.facebook_comments_post_uuid),
        col(fields.facebook_comments_parent_uuid),
        conv_timestamp("created_time").alias(fields.facebook_comments_created_at),
        col("anonymized_message").alias(fields.facebook_comments_text),
        col("permalink_url").alias(fields.facebook_comments_permalink_url),
        col(fields.reactions_struct_field),
        struct(
            col("_source_key_id").alias("id"),
            col("_source_key_post_id").alias("post_id"),
            col("_source_key_parent_id").alias("parent_id"),
        ).alias(fields.facebook_comments_source_keys),
        col(fields.bronze_timestamp_field),
    )

    return comments_df
